<?php
/*
Plugin Name: Advanced Content Sitemap Generator
Description: Generates sitemaps for tags, images, pages, categories, posts, and titles. Creates XML, TXT, HTM, and ZIP files for each type.
Version: 1.0
Author: Mbsmgroup
*/

class Advanced_Content_Sitemap_Generator {

    private string $sitemap_folder = 'advanced-sitemaps'; // ??? ?????? ??????
    private string $file_name = 'sitemap'; // ??? ????? ?????????
    private int $limit = 3000; // ???? ?????? ?????????

    public function __construct() {
        add_action('admin_menu', [$this, 'add_admin_menu']);
    }

    /**
     * Adds an admin menu page.
     */
    public function add_admin_menu() {
        add_menu_page(
            'Advanced Sitemap Generator',  // Page title
            'Advanced Sitemap',  // Menu title
            'manage_options', // Capability
            'advanced-sitemap',   // Menu slug
            [$this, 'render_admin_page'], // Callback function
            'dashicons-admin-site-alt3', // Icon
            20  // Position
        );
    }

    /**
     * Renders the admin page.
     */
    public function render_admin_page() {
        // Check if the form is submitted
        if (isset($_POST['generate_sitemap'])) {
            $this->generate_sitemap();
            echo '<div class="notice notice-success"><p>Sitemap files have been successfully generated. Check the table below:</p></div>';
        }

        $upload_dir = wp_upload_dir();
        $sitemap_folder_path = $upload_dir['basedir'] . '/' . $this->sitemap_folder . '/';

        ?>
        <div class="wrap">
            <h1>Advanced Sitemap Generator</h1>
            <p>Click the button below to generate sitemaps for tags, images, pages, categories, posts, and titles.</p>
            <form method="post" action="">
                <input type="hidden" name="generate_sitemap" value="1">
                <?php submit_button('Generate Sitemap Files'); ?>
            </form>

            <?php
            // Display the table for generated files after the generation
            if (isset($_POST['generate_sitemap'])) {
                $files = glob($sitemap_folder_path . '*');
                if (!empty($files)) {
                    echo '<h2>Generated Files</h2>';
                    echo '<table class="wp-list-table widefat fixed striped">';
                    echo '<thead><tr><th>File Name</th><th>File URL</th><th>File Size</th><th>Last Modified</th></tr></thead>';
                    echo '<tbody>';
                    foreach ($files as $file) {
                        $filename = basename($file);
                        $file_url = esc_url($upload_dir['baseurl'] . '/' . $this->sitemap_folder . '/' . $filename);
                        $file_size = size_format(filesize($file));
                        $file_modified = date('F d, Y H:i:s', filemtime($file));
                        echo '<tr>';
                        echo '<td>' . esc_html($filename) . '</td>';
                        echo '<td><a href="' . $file_url . '" target="_blank">' . esc_html($filename) . '</a></td>';
                        echo '<td>' . esc_html($file_size) . '</td>';
                        echo '<td>' . esc_html($file_modified) . '</td>';
                        echo '</tr>';
                    }
                    echo '</tbody></table>';
                }
            }
            ?>
        </div>
        <?php
    }

    /**
     * Generates the sitemap files.
     */
    public function generate_sitemap() {
        $upload_dir = wp_upload_dir();
        $sitemap_folder_path = $upload_dir['basedir'] . '/' . $this->sitemap_folder . '/';

        // ????? ?????? ??? ?? ??? ???????
        if (!file_exists($sitemap_folder_path)) {
            wp_mkdir_p($sitemap_folder_path);
        }

        // ??? ???? ??????? ??????? ?? ??????
        $this->delete_old_files($sitemap_folder_path);

        // ????? ????? Sitemap ??? ??? ?? ????????
        $sitemaps = [
            'tags' => $this->get_tags_urls(),
            'images' => $this->get_images_urls(),
            'pages' => $this->get_pages_urls(),
            'categories' => $this->get_categories_urls(),
            'post' => $this->get_posts_urls(),
            'title' => $this->get_titles_urls(), // ????? ??? title
        ];

        foreach ($sitemaps as $type => $urls) {
            $this->create_files($urls, $sitemap_folder_path, $type);
        }
    }

    /**
     * Deletes all files in the specified folder.
     *
     * @param string $folder_path The path to the folder.
     */
    private function delete_old_files(string $folder_path): void {
        if (is_dir($folder_path)) {
            $files = glob($folder_path . '*'); // ??? ???? ??????? ?? ??????
            foreach ($files as $file) {
                if (is_file($file)) {
                    unlink($file); // ??? ?????
                }
            }
        }
    }

    /**
     * Creates multiple files (XML, TXT, HTM, ZIP) for a given type.
     *
     * @param array $urls The list of URLs to include in the files.
     * @param string $folder_path The path to the folder.
     * @param string $type The type of content (e.g., tags, images, etc.).
     */
    private function create_files(array $urls, string $folder_path, string $type): void {
        // ????? ??? XML
        $xml_content = $this->generate_xml_content($urls);
        file_put_contents($folder_path . $type . '.xml', $xml_content);

        // ????? ??? TXT
        $txt_content = '';
        foreach ($urls as $url) {
            $txt_content .= $url['loc'] . PHP_EOL;
        }
        file_put_contents($folder_path . $type . '.txt', $txt_content);

        // ????? ??? HTM
        $htm_content = '<!DOCTYPE html><html><head><title>' . ucfirst($type) . ' Sitemap</title></head><body><ul>';
        foreach ($urls as $url) {
            $htm_content .= '<li><a href="' . esc_url($url['loc']) . '">' . esc_html($url['loc']) . '</a></li>';
        }
        $htm_content .= '</ul></body></html>';
        file_put_contents($folder_path . $type . '.htm', $htm_content);

        // ????? ??? ZIP
        $zip = new ZipArchive();
        if ($zip->open($folder_path . $type . '.zip', ZipArchive::CREATE) === TRUE) {
            $zip->addFromString($type . '.xml', $xml_content);
            $zip->addFromString($type . '.txt', $txt_content);
            $zip->addFromString($type . '.htm', $htm_content);
            $zip->close();
        }
    }

    /**
     * Generates XML content for the sitemap.
     *
     * @param array $urls The list of URLs to include in the sitemap.
     * @return string The XML content.
     */
    private function generate_xml_content(array $urls): string {
        $xml = '<?xml version="1.0" encoding="UTF-8"?>' . PHP_EOL;
        $xml .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . PHP_EOL;

        foreach ($urls as $url) {
            $xml .= '<url>' . PHP_EOL;
            $xml .= '<loc>' . esc_url($url['loc']) . '</loc>' . PHP_EOL;
            if (isset($url['lastmod'])) {
                $xml .= '<lastmod>' . esc_html($url['lastmod']) . '</lastmod>' . PHP_EOL;
            }
            $xml .= '</url>' . PHP_EOL;
        }

        $xml .= '</urlset>';
        return $xml;
    }

    /**
     * Fetches URLs for tags.
     *
     * @return array The list of tag URLs.
     */
    private function get_tags_urls(): array {
        $tags = get_tags(['number' => $this->limit]);
        $urls = [];
        foreach ($tags as $tag) {
            $urls[] = ['loc' => get_tag_link($tag)];
        }
        return $urls;
    }

    /**
     * Fetches URLs for images.
     *
     * @return array The list of image URLs.
     */
    private function get_images_urls(): array {
        $attachments = get_posts([
            'post_type' => 'attachment',
            'post_mime_type' => 'image',
            'numberposts' => $this->limit,
            'orderby' => 'date',
            'order' => 'DESC',
        ]);
        $urls = [];
        foreach ($attachments as $attachment) {
            $urls[] = ['loc' => wp_get_attachment_url($attachment->ID), 'lastmod' => get_the_date('c', $attachment->ID)];
        }
        return $urls;
    }

    /**
     * Fetches URLs for pages.
     *
     * @return array The list of page URLs.
     */
    private function get_pages_urls(): array {
        $pages = get_pages(['number' => $this->limit, 'orderby' => 'date', 'order' => 'DESC']);
        $urls = [];
        foreach ($pages as $page) {
            $urls[] = ['loc' => get_page_link($page), 'lastmod' => get_the_date('c', $page->ID)];
        }
        return $urls;
    }

    /**
     * Fetches URLs for categories.
     *
     * @return array The list of category URLs.
     */
    private function get_categories_urls(): array {
        $categories = get_categories(['number' => $this->limit]);
        $urls = [];
        foreach ($categories as $category) {
            $urls[] = ['loc' => get_category_link($category)];
        }
        return $urls;
    }

    /**
     * Fetches URLs for posts.
     *
     * @return array The list of post URLs.
     */
    private function get_posts_urls(): array {
        $posts = get_posts(['numberposts' => $this->limit, 'orderby' => 'date', 'order' => 'DESC']);
        $urls = [];
        foreach ($posts as $post) {
            $urls[] = ['loc' => get_permalink($post), 'lastmod' => get_the_date('c', $post->ID)];
        }
        return $urls;
    }

    /**
     * Fetches titles for posts.
     *
     * @return array The list of post titles.
     */
    private function get_titles_urls(): array {
        $posts = get_posts(['numberposts' => $this->limit, 'orderby' => 'date', 'order' => 'DESC']);
        $urls = [];
        foreach ($posts as $post) {
            $urls[] = ['loc' => get_the_title($post)];
        }
        return $urls;
    }
}

new Advanced_Content_Sitemap_Generator();